# Written by Robert C. Kirby
# Copyright 2005 by The University of Chicago
# Distributed under the LGPL license
# This work is partially supported by the US Department of Energy
# under award number DE-FG02-04ER25650

# Last modified 3 February 2005

import Numeric

class Functional( object ):
    def __init__( self , U , a ):
        self.U, self.a = U, Numeric.asarray( a )
        return
    def __call__( self , p ):
        # need to add type checking to confirm that
        # p is a member of self.U
        return Numeric.dot( self.a , p.dof )
    def __add__( self , other ):
	# operations on functionals are cute because now
	# they are implemented by vector arithmetic
	if not self.U is other.U:
	    raise RuntimeError, "type mismatch in adding functionals."
	return Functional( self.U , self.dof + other.dof )

# Here is the new bottleneck -- I will have to call
# eval_all for every point of the dual basis.
# I need a function that will create PointEvaluation
# in batches.

class PointEvaluation( Functional ):
    def __init__( self , U , pt ):
        Functional.__init__( self , U , U.eval_all( pt ) )
        return

# Also, how do I create derivatives?
# I want to evaluate the derivatives at certain points.
# That means evaluating the derivatives 
# Generally, I don't have to do this but 6 or twelve times,
# so I'm not optimizing now.
class PointDerivative( Functional ):
    def __init__( self , U , pt , i ):
        Functional.__init__( self , U , [ u.deriv(i)(pt) for u in U ] )
        return

# Integral moments on the interior are not hard
# l_u (v) = int( u * v ) means we have to dot the vector of
# coefficients of the function u with the vector of coefficients
# of the input function v, provided that our orthogonal expansions are
# all normalized
#
# This is just Parseval's relation -- L2 inner product of functions
# represented in an orthonormal basis is just the dot product of
# the coefficients in that orthonormal basis

class IntegralMoment( Functional ):
    def __init__( self , U , p ):
        Functional.__init__( self , U , p.dof )
        return
